package cn.tee3.n2m.ui.util;

import android.os.Environment;
import android.util.Log;

import net.ossrs.yasea.SrsEncoder;
import net.ossrs.yasea.SrsFlvMuxer;
import net.ossrs.yasea.SrsMp4Muxer;
import net.ossrs.yasea.rtmp.RtmpPublisher;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import cn.tee3.avd.ErrorCode;
import cn.tee3.avd.FakeVideoCapturer;
import cn.tee3.avd.FakeVideoCapturer.FourccType;
import cn.tee3.avd.MAudio;
import cn.tee3.avd.MVideo;
import cn.tee3.avd.Room;
import cn.tee3.n2m.ui.fragment.VideosFragment;

/**
 * Created by Administrator on 2016/6/6.
 */
public class CliRtmpPusher {
    private static final String TAG = "CliRtmpPusher";
    MAudio maudio;
    MVideo mvideo;
    VideosFragment mainui;
    private String rtmpUrl = "rtmp://121.41.102.126/live/";
//    private String rtmpUrl = "rtmp://publish.3tee.com.cn/ucloud/1234567";
    private String recPath = Environment.getExternalStorageDirectory().getPath() + "/test.mp4";
    private boolean ispublished = false;
    private boolean testPreview = false;
    private boolean testFakecamera = false;
    FakeVideoCapturer mFakeVCapturer;
    ///
    private float x = 0.7f, w = 0.3f, h = 0.3f;
    private static final float curr_y_begin = 0.1f;
    private float curr_y = curr_y_begin;
    List<String>  curr_overlays = new ArrayList<String>();
    public void initPusher(VideosFragment mainui) {
        this.mainui = mainui;
    }
    public boolean needOverlayVideo() {
        return curr_y <(1.0f-h);
    }
    public boolean startPublish(String videoDeviceId) {
        Log.i(TAG, "startPublish, begin...");
        if (ispublished) {
            return true;
        }
        this.mainui.setOnCameraStatusListener(this.camStatus);
        ispublished = true;
        Room room = Room.obtain(N2MSetting.getInstance().getRoomId());
        if (null == room) {
            Log.e(TAG, "startPublish, obtain room failed. roomId=" + N2MSetting.getInstance().getRoomId());
            return false;
        }
        maudio = MAudio.getAudio(room);
        mvideo = MVideo.getVideo(room);
        if (null == maudio || null == mvideo) {
            Log.e(TAG, "startPublish, obtain MAudio MVideo failed.");
            return false;
        }

        if (testFakecamera) {
            this.mFakeVCapturer = FakeVideoCapturer.Create(new FakeVideoCapturer.Listener() {
                @Override
                public void onStart() {
                    Log.i(TAG, "FakeVideoCapturer onStart");
                }
                @Override
                public void onStop() {
                    Log.i(TAG, "FakeVideoCapturer onStop");
                }
            }, FourccType.ft_I420, false);
            MVideo.Camera fakeCam = new MVideo.Camera("fakecapturertest", "fake camera");
            int ret = mvideo.publishLocalCamera(fakeCam, mFakeVCapturer);
            //int ret = mvideo.previewLocalCamera(fakeCam,mFakeVCapturer,null);
            if (ret != ErrorCode.AVD_OK) {
                Log.e(TAG, "startPublish,publishLocalCamera with fakecamera. ret="+ret);
            }
        }

        String curUrl = rtmpUrl + room.getRoomId(); // + getRandomAlphaString(10);
        //curUrl = "rtmp://pili-live-rtmp.pilitest.qiniucdn.com:1935/pilitest/575aca6f5e77b08bdc01a5be";
        Log.i(TAG, "startPublish, curUrl:" + curUrl);
        try {
            flvMuxer.start(curUrl);
        } catch (IOException e) {
            Log.e(TAG, "startPublish, start FLV muxer failed.");
            e.printStackTrace();
            return false;
        }
        flvMuxer.setVideoResolution(mEncoder.VCROP_WIDTH, mEncoder.VCROP_HEIGHT);
        startEncoder();

        room.setOption(Room.Option.ro_audio_mixerdata_callback_buffered, "true");
        room.setOption(Room.Option.ro_video_mixerdata_callback_format, String.valueOf(mEncoder.getVideoColorFormat()));

        mvideo.setMixerDataListener(this.onVideoData);
        boolean hasMainVideo = false;
        if (testPreview) {
            MVideo.Camera item = mvideo.getCamera(mvideo.getCurrentCameraType());
            mvideo.setMixerMainVideo(item.getId());
            hasMainVideo = true;
        }
        else {
            List<MVideo.Camera> cameras = mvideo.getPublishedCameras();
            if (null != videoDeviceId) {
                mvideo.setMixerMainVideo(videoDeviceId);
            }
            else {
                for (MVideo.Camera item : cameras) {
                    if (mvideo.isSelfDevice(item.getId())) {
                        mvideo.setMixerMainVideo(item.getId());
                        hasMainVideo = true;
                        break;
                    }
                }
            }
            if (hasMainVideo) {
                for (MVideo.Camera item : cameras) {
                    if (curr_y >(1.0f-h)) {
                        break;
                    }
                    if (!mvideo.isSelfDevice(item.getId())) {
                        mvideo.addMixerOverlayVideo(item.getId(), 0, x,curr_y,w,h);
                        curr_overlays.add(item.getId());
                        curr_y += h;
                    }
                }
            }
            else {
                Log.e(TAG, "startPublish, no main video, cann't start live.");
                mEncoder.stop();
                return false;
            }
        }

        maudio.setMixerDataListener(this.onAudioData);
        Log.i(TAG, "startPublish, begin ok");
        return true;
    }

    public void stopPublish() {
        if (!ispublished) {
            return ;
        }
        Log.i(TAG, "stopPublish,begin...");
        this.mainui.setOnCameraStatusListener(null);
        curr_y = curr_y_begin;
        curr_overlays.clear();
        if (null != maudio) {
            maudio.setMixerDataListener(null);
        }
        if (null != mvideo) {
            mvideo.clearMixerVideos();
            mvideo.setMixerDataListener(null);
        }
        flvMuxer.stop();
        mp4Muxer.stop();
        stopEncoder();
        ispublished = false;

        if (null != this.mFakeVCapturer) {
            FakeVideoCapturer.destoryCapturer(this.mFakeVCapturer);
            this.mFakeVCapturer = null;
        }
        Log.i(TAG, "stopPublish,end");
    }

    private static String getRandomAlphaString(int length) {
        String base = "abcdefghijklmnopqrstuvwxyz";
        Random random = new Random();
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < length; i++) {
            int number = random.nextInt(base.length());
            sb.append(base.charAt(number));
        }
        return sb.toString();
    }

    private static String getRandomAlphaDigitString(int length) {
        String base = "abcdefghijklmnopqrstuvwxyz0123456789";
        Random random = new Random();
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < length; i++) {
            int number = random.nextInt(base.length());
            sb.append(base.charAt(number));
        }
        return sb.toString();
    }

    private VideosFragment.OnRemoteVideoStatus camStatus = new VideosFragment.OnRemoteVideoStatus() {
        @Override
        public void onVideoSubscribeStatus(boolean subed, String fromId) {
            if (mvideo.isSelfDevice(fromId)) {
                return ;
            }
            if (subed) {
                if (curr_y > (1.0f-h)) {
                   return ;
                }
                mvideo.addMixerOverlayVideo(fromId, 0, x,curr_y,w,h);
                curr_overlays.add(fromId);
                curr_y += h;
            }
            else {
                if (!curr_overlays.contains(fromId)) {
                    return ;
                }
                mvideo.removeOverlayVideo(fromId);
                curr_overlays.remove(fromId);
                curr_y = curr_y_begin;
                for (String item : curr_overlays) {
                    mvideo.updateMixerOverlayVideo(item, 0, x, curr_y, w, h);
                    curr_y += h;
                }
            }
        }
    };
    private SrsFlvMuxer flvMuxer = new SrsFlvMuxer(new RtmpPublisher.EventHandler() {
        @Override
        public void onRtmpConnecting(String msg) {
            Log.i(TAG, "onRtmpConnecting, msg" + msg);
            //ToastUtil.showToast(msg);
        }

        @Override
        public void onRtmpConnected(String msg) {
            Log.i(TAG, "onRtmpConnected, msg" + msg);
           // ToastUtil.showToast(msg);
        }

        @Override
        public void onRtmpVideoStreaming(String msg) {
            //Log.i(TAG, "onRtmpVideoStreaming, msg" +msg);
        }

        @Override
        public void onRtmpAudioStreaming(String msg) {
            //Log.i(TAG, "onRtmpAudioStreaming, msg" +msg);
        }

        @Override
        public void onRtmpStopped(String msg) {
            Log.i(TAG, "onRtmpStopped, msg" + msg);
            //ToastUtil.showToast(msg);
        }

        @Override
        public void onRtmpDisconnected(String msg) {
            Log.i(TAG, "onRtmpDisconnected, msg" + msg);
           // ToastUtil.showToast(msg);
        }

        @Override
        public void onRtmpOutputFps(final double fps) {
            Log.i(TAG, String.format("onRtmpOutputFps, Output Fps: %f", fps));
        }
    });
    private SrsMp4Muxer mp4Muxer = new SrsMp4Muxer(new SrsMp4Muxer.EventHandler() {
        @Override
        public void onRecordPause(String msg) {
            Log.i(TAG, "onRecordPause, msg" + msg);
        }

        @Override
        public void onRecordResume(String msg) {
            Log.i(TAG, "onRecordResume, msg" + msg);
        }

        @Override
        public void onRecordStarted(String msg) {
            Log.i(TAG, "onRecordStarted, msg" + msg);
        }

        @Override
        public void onRecordFinished(String msg) {
            Log.i(TAG, "onRecordFinished, msg" + msg);
        }
    });
    private SrsEncoder mEncoder = new SrsEncoder(flvMuxer, mp4Muxer);

    private MVideo.MixerDataListener onVideoData = new MVideo.MixerDataListener() {
        private int mWidth;
        private int mHeight;
        @Override
        public void onVideoSize(int width, int height) {
            Log.i("onVideoSize", "setSize:" + width + " x " + height);
            mWidth = width;
            mHeight = height;
        }
        @Override
        public void onVideoData(long pts, byte[] data, int len) {
            if (testFakecamera) {
                if (mFakeVCapturer.isRunning()) {
                    mFakeVCapturer.inputCapturedFrame(0,mWidth,mHeight,data,len,0, false);
                }
            }
            else {
                mEncoder.onGetYuvFrame(data);
            }
        }
    };
    private MAudio.MixerDataListener onAudioData = new MAudio.MixerDataListener() {
        @Override
        public void onAudioParam(int sampleRate, int channels) {
            Log.i("onAudioParam", "sampleRate:" + sampleRate + ",channels=" + channels);
        }
        @Override
        public void onAudioData(long pts, byte[] data, int len) {
            //Log.i("onAudioData", "pts:" + pts + ",len=" + len);
            //Log.i("onAudioData", "data:" + data[0] + " "+ data[1] + " " + data[2]);
            if (testFakecamera) {

            }
            else {
                mEncoder.onGetPcmFrame(data, len);
            }
        }
    };

    private void startEncoder() {
        int ret = mEncoder.start();
        if (ret < 0) {
            return;
        }
        return ;
    }
    private void stopEncoder() {
        mEncoder.stop();
    }


}
